#!/usr/bin/env python3
# encoding: utf-8
#
# This file is a part of the CaosDB Project.
#
# Copyright (C)  2023 Research Group Biomedical Physics,
# Max-Planck-Institute for Dynamics and Self-Organization Göttingen
# Alexander Schlemmer <alexander.schlemmer@ds.mpg.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#

"""
Unit test functions for the transformer feature of the scanner.

Currently, this is under development.
See: https://gitlab.indiscale.com/caosdb/src/caosdb-crawler/-/issues/107
"""

from functools import partial
from pathlib import Path
from tempfile import NamedTemporaryFile
from unittest.mock import MagicMock, Mock, patch

import caosdb as db
import pytest
import yaml
from caoscrawler.scanner import (create_converter_registry, load_definition,
                                 scan_directory, scan_structure_elements)
from caoscrawler.structure_elements import (DictElement, DictListElement,
                                            DictTextElement, File)
from pytest import raises

UNITTESTDIR = Path(__file__).parent


def test_simple_transformer():
    """
    Test the correct list of returned records by the scanner using the
    scifolder example from the article.
    """

    records = scan_directory(UNITTESTDIR / "test_directories" / "test_transformers",
                             UNITTESTDIR / "test_directories" / "test_transformers" /
                             "cfood.yml")

    for r in records:
        if r.parents[0].name == "DayFolder":
            assert r.get_property("Day") is not None
            assert r.get_property("DayShort") is not None
            assert r.get_property("DayShort").value != "$day_short"
            if r.get_property("DayShort").value == "Unk":
                # This unkown folder should not lead to a replacement
                assert r.get_property("Day").value == "Unk"
                assert r.get_property("DaySplit").value == ["Unk"]
            elif r.get_property("DayShort").value == "Mon":
                assert r.get_property("Day").value == "Monday"
                assert r.get_property("DaySplit").value == ["M", "n"]
            elif r.get_property("DayShort").value == "Tue":
                assert r.get_property("Day").value == "Tuesday"
                assert r.get_property("DaySplit").value == ["Tue"]
            else:
                # unexpected occurence of a short form, something wrong with test directories
                assert False
        elif r.parents[0].name == "Number":
            assert r.get_property("num") is not None
            assert r.get_property("num").value == "'12345 5 '"
        else:
            # unkown error, something wrong with test directories
            assert False
